package com.example.angelus9110.routes4riders;

import android.Manifest;
import android.app.IntentService;
import android.content.ContentValues;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.v4.app.ActivityCompat;
import android.util.Log;

import com.example.angelus9110.routes4riders.sqlite.DbHelper;
import com.example.angelus9110.routes4riders.sqlite.PointContract;
import com.example.angelus9110.routes4riders.sqlite.RouteContract;

public class LocationService extends IntentService {
    public static final String TAG = LocationService.class.getSimpleName();

    public static final int INTERVAL = 10000;
    public static final float DISTANCE = 2f;
    private LocationManager locationManager = null;
    private boolean runFlag = false;
    DbHelper dbHelper;

    public LocationService() {
        super(TAG);
    }

    MyLocationListener myLocationListener = new MyLocationListener(LocationManager.GPS_PROVIDER);

    @Override
    protected void onHandleIntent(@Nullable Intent intent) {
        Log.d(TAG, "onHandleIntent");
        runFlag = true;

        initializeLocationManager();
        try {
            if (ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
                Log.e(TAG, "No se han dado permisos de ubicación. No se guardará la ruta.");
                return;
            }
            locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 5000, DISTANCE, myLocationListener);
        } catch (SecurityException e) {
            Log.d(TAG, "No se puede obtener la localizacion");
        } catch (IllegalArgumentException e) {
            Log.d(TAG, "No existe el proveedor GPS");
        }

        // Insertar ruta
        long routeTimestamp = System.currentTimeMillis() / 1000;
        ContentValues routeValues = new ContentValues();
        routeValues.put(RouteContract.Column.CREATED_AT, routeTimestamp);
        getContentResolver().insert(RouteContract.CONTENT_URI, routeValues);

        while (runFlag) {
            Location location = locationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);
            // Inserto puntos en BD

            long pointTimestamp = location.getTime();
            double lat = location.getLatitude();
            double lon = location.getLongitude();

            ContentValues pointValues = new ContentValues();
            pointValues.put(PointContract.Column.CREATED_AT, pointTimestamp);
            pointValues.put(PointContract.Column.LAT, lat);
            pointValues.put(PointContract.Column.LON, lon);
            pointValues.put(PointContract.Column.ROUTE_ID, routeTimestamp);

            getContentResolver().insert(PointContract.CONTENT_URI, pointValues);
            try {
                Thread.sleep(INTERVAL);
            } catch (InterruptedException e) {
                runFlag = false;
            }
        }

    }

    private void initializeLocationManager() {
        Log.d(TAG, "initializeLocationManager");
        if (locationManager == null)
            locationManager = (LocationManager) getApplicationContext().getSystemService(LOCATION_SERVICE);
    }

    @Override
    public void onCreate() {
        super.onCreate();
        Log.d(TAG, "onCreated");

        dbHelper = new DbHelper(this);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        Log.d(TAG, "onDestroyed");
        runFlag = false;
        if (locationManager != null) locationManager.removeUpdates(myLocationListener);
    }

    private class MyLocationListener implements LocationListener {

        public MyLocationListener(String gpsProvider) {
            Log.d(TAG, "LocationListener: " + gpsProvider);
        }

        @Override
        public void onLocationChanged(Location location) {
            Log.d(TAG, "onLocationChanged: " + location);

        }

        @Override
        public void onStatusChanged(String gpsProvider, int i, Bundle bundle) {
            Log.d(TAG, "onStatusChanged: " + gpsProvider);
        }

        @Override
        public void onProviderEnabled(String gpsProvider) {
            Log.d(TAG, "onProviderEnabled: " + gpsProvider);
        }

        @Override
        public void onProviderDisabled(String gpsProvider) {
            Log.d(TAG, "onProviderDisabled: " + gpsProvider);
        }
    }
}
