package com.example.angelus9110.routes4riders.sqlite;

import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

import java.util.Locale;

public class DbHelper extends SQLiteOpenHelper {
    private static String TAG = DbHelper.class.getSimpleName();

    private static final long TIMESTAMP2000 = 946684800;

    public DbHelper(Context context) {
        super(context, SQLiteContract.DB_NAME, null, SQLiteContract.DB_VERSION);
    }

    @Override
    public void onCreate(SQLiteDatabase sqLiteDatabase) {
        Log.d(TAG, "onCreated");

        creaTablaRuta(sqLiteDatabase);
        insertaDatosEjemploRuta(sqLiteDatabase);

        creaTablaPunto(sqLiteDatabase);
        insertaDatosEjemploPunto(sqLiteDatabase);

        creaTablaUsuario(sqLiteDatabase);

        creaTablaMoto(sqLiteDatabase);
    }

    private void insertaDatosEjemploRuta(SQLiteDatabase sqLiteDatabase) {
        String sql = String.format(Locale.ENGLISH, "INSERT INTO %s (%s) VALUES (%d)",
                RouteContract.TABLE,
                RouteContract.Column.CREATED_AT,
                TIMESTAMP2000);
        Log.d(TAG, sql);
        sqLiteDatabase.execSQL(sql);
    }

    private void creaTablaRuta(SQLiteDatabase sqLiteDatabase) {
        String sql = String.format("CREATE TABLE %s (%s INTEGER PRIMARY KEY)",
                RouteContract.TABLE,
                RouteContract.Column.CREATED_AT);

        sqLiteDatabase.execSQL(sql);
    }

    private void creaTablaPunto(SQLiteDatabase sqLiteDatabase) {
        String sql = String.format("CREATE TABLE %s (%s INTEGER PRIMARY KEY, %s REAL, %s REAL, %s INTEGER, " +
                        "FOREIGN KEY(%s) REFERENCES %s(%s))",
                PointContract.TABLE,
                PointContract.Column.CREATED_AT,
                PointContract.Column.LAT,
                PointContract.Column.LON,
                PointContract.Column.ROUTE_ID,
                PointContract.Column.ROUTE_ID,
                RouteContract.TABLE,
                RouteContract.Column.CREATED_AT);

        sqLiteDatabase.execSQL(sql);
    }

    private void creaTablaUsuario(SQLiteDatabase sqLiteDatabase) {
        String sql = String.format("CREATE TABLE %s (%s INTEGER PRIMARY KEY AUTOINCREMENT, %s TEXT, %s TEXT, %s TEXT, %s TEXT, " +
                        "FOREIGN KEY(%s) REFERENCES %s(%s), FOREIGN KEY(%s) REFERENCES %s(%s))",
                UserContract.TABLE,
                UserContract.Column.ID,
                UserContract.Column.USER,
                UserContract.Column.EMAIL,
                UserContract.Column.MARCA,
                UserContract.Column.MODELO,
                UserContract.Column.MARCA,
                MotoContract.TABLE,
                MotoContract.Column.MARCA_MOTO,
                UserContract.Column.MODELO,
                MotoContract.TABLE,
                MotoContract.Column.MODELO_MOTO);

        sqLiteDatabase.execSQL(sql);
    }

    private void creaTablaMoto(SQLiteDatabase sqLiteDatabase) {
        String sql = String.format("CREATE TABLE %s (%s INTEGER PRIMARY KEY AUTOINCREMENT, %s TEXT, %s TEXT, UNIQUE(%s, %s))",
                MotoContract.TABLE,
                MotoContract.Column.ID,
                MotoContract.Column.MARCA_MOTO,
                MotoContract.Column.MODELO_MOTO,
                MotoContract.Column.MARCA_MOTO,
                MotoContract.Column.MODELO_MOTO);

        sqLiteDatabase.execSQL(sql);
    }


    @Override
    public void onUpgrade(SQLiteDatabase sqLiteDatabase, int oldVersion, int newVersion) {
        //Aqui irian las sentencias del tipo ALTER TABLE, do momento lo hacemos mas sencillo...
        //Borramos la vieja base de datos
        //String sentencia = "DROP TABLE IF EXISTS";
        sqLiteDatabase.execSQL("DROP TABLE IF EXISTS " + PointContract.TABLE);
        sqLiteDatabase.execSQL("DROP TABLE IF EXISTS " + RouteContract.TABLE);
        sqLiteDatabase.execSQL("DROP TABLE IF EXISTS " + MotoContract.TABLE);
        sqLiteDatabase.execSQL("DROP TABLE IF EXISTS " + UserContract.TABLE);

        //Creamos la nueva base de datos
        onCreate(sqLiteDatabase);
        Log.d(TAG, "onUpgrade");
    }

    private void insertaDatosEjemploPunto(SQLiteDatabase sqLiteDatabase) {
        String sqlTemplate = "INSERT INTO %s (%s, %s, %s, %s) VALUES (%f, %f, %d, %d)";
        String sql;

        sql = String.format(Locale.ENGLISH, sqlTemplate,
                PointContract.TABLE,
                PointContract.Column.LAT,
                PointContract.Column.LON,
                PointContract.Column.ROUTE_ID,
                PointContract.Column.CREATED_AT,
                41.652111, -4.729136, TIMESTAMP2000, 350);
        sqLiteDatabase.execSQL(sql);

        sql = String.format(Locale.ENGLISH, sqlTemplate,
                PointContract.TABLE,
                PointContract.Column.LAT,
                PointContract.Column.LON,
                PointContract.Column.ROUTE_ID,
                PointContract.Column.CREATED_AT,
                41.651654, -4.729056, TIMESTAMP2000, 360);
        sqLiteDatabase.execSQL(sql);

        sql = String.format(Locale.ENGLISH, sqlTemplate,
                PointContract.TABLE,
                PointContract.Column.LAT,
                PointContract.Column.LON,
                PointContract.Column.ROUTE_ID,
                PointContract.Column.CREATED_AT,
                41.652227, -4.728093, TIMESTAMP2000, 370);
        sqLiteDatabase.execSQL(sql);
    }

}
