package com.example.angelus9110.routes4riders.sqlite;

import android.content.ContentProvider;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.UriMatcher;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.net.Uri;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.text.TextUtils;
import android.util.Log;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

public class MotoProvider extends ContentProvider {
    private static final String TAG = PointContract.class.getSimpleName();
    private DbHelper dbHelper;
    private static final UriMatcher sUriMatcher;

    static {
        sUriMatcher = new UriMatcher(UriMatcher.NO_MATCH);
        sUriMatcher.addURI(MotoContract.AUTHORITY, MotoContract.TABLE, MotoContract.MOTO_DIR);
        sUriMatcher.addURI(MotoContract.AUTHORITY, MotoContract.TABLE + "/#", MotoContract.MOTO_ITEM);
    }

    @Override
    public boolean onCreate() {
        dbHelper = new DbHelper(getContext());
        Log.d(TAG, "onCreated");


        InputStream scriptSql = null;
        SQLiteDatabase db = dbHelper.getReadableDatabase();
        try {
            scriptSql = getContext().getAssets().open("import.sql");
            if (scriptSql != null){
                db.beginTransaction();
                BufferedReader reader = new BufferedReader(new InputStreamReader(scriptSql));
                String line = reader.readLine();
                while (!TextUtils.isEmpty(line)){
                    db.execSQL(line);
                    line = reader.readLine();
                }
                db.setTransactionSuccessful();
            }

        } catch (Exception e){
            Log.d(TAG, "Fichero no encontrado");
        } finally {
            db.endTransaction();
            if(scriptSql != null){
                try {
                    scriptSql.close();
                } catch (IOException eIO){
                    Log.d(TAG, "Fichero sin contenido");
                }
            }
        }
        return true;
    }

    @Nullable
    @Override
    public Cursor query(@NonNull Uri uri, @Nullable String[] strings, @Nullable String s, @Nullable String[] strings1, @Nullable String s1) {
        String where;

        switch (sUriMatcher.match(uri)) {
            case MotoContract.MOTO_DIR:
                where = s;
                break;
            case MotoContract.MOTO_ITEM:
                long id = ContentUris.parseId(uri);
                where = MotoContract.Column.ID
                        + "="
                        + id
                        + (TextUtils.isEmpty(s) ? "" : " and (" + s + ")");
                break;
            default:
                throw new IllegalArgumentException("Uri incorrecta: " + uri);
        }
        String orderBy = TextUtils.isEmpty(s1) ? MotoContract.DEFAULT_SORT : s1;
        SQLiteDatabase db = dbHelper.getReadableDatabase();
        Cursor cursor = db.query(MotoContract.TABLE, strings, where, strings1, null, null, orderBy);
        cursor.setNotificationUri(getContext().getContentResolver(), uri);
        Log.d(TAG, "Registros recuperados: " + cursor.getCount());
        return cursor;
    }

    @Nullable
    @Override
    public String getType(@NonNull Uri uri) {
        String type;
        switch (sUriMatcher.match(uri)) {
            case MotoContract.MOTO_DIR:
                type = "vnd.android.cursor.dir/vnd.com.example.angelus9110.routes4riders.sqlite.provider.moto";
                return type;
            case MotoContract.MOTO_ITEM:
                type = "vnd.android.cursor.item/vnd.com.example.angelus9110.routes4riders.sqlite.provider.moto";
                return type;
            default:
                throw new IllegalArgumentException("Uri incorrecta: " + uri);
        }
    }

    @Nullable
    @Override
    public Uri insert(@NonNull Uri uri, @Nullable ContentValues contentValues) {
        Uri ret = null;

        // Aseguramos que la URI es correcta
        if (sUriMatcher.match(uri) != MotoContract.MOTO_DIR)
            throw new IllegalArgumentException("Uri incorrecta: " + uri);

        SQLiteDatabase db = dbHelper.getWritableDatabase();
        long rowId = db.insertWithOnConflict(MotoContract.TABLE, null, contentValues, SQLiteDatabase.CONFLICT_IGNORE);

        if (rowId != -1) {
            long id = contentValues.getAsLong(MotoContract.Column.ID);
            ret = ContentUris.withAppendedId(uri, id);
            Log.d(TAG, "Uri insertada: " + ret);
            getContext().getContentResolver().notifyChange(uri, null);
        }
        return ret;
    }

    @Override
    public int delete(@NonNull Uri uri, @Nullable String s, @Nullable String[] strings) {
        String where;

        switch (sUriMatcher.match(uri)) {
            case MotoContract.MOTO_DIR:
                where = s;
                break;
            case MotoContract.MOTO_ITEM:
                long id = ContentUris.parseId(uri);
                where = MotoContract.Column.ID
                        + "="
                        + id
                        + (TextUtils.isEmpty(s) ? "" : " and (" + s + ")");
                break;
            default:
                throw new IllegalArgumentException("Uri incorrecta: " + uri);
        }
        SQLiteDatabase db = dbHelper.getWritableDatabase();
        int ret = db.delete(MotoContract.TABLE, where, strings);
        if (ret > 0)
            getContext().getContentResolver().notifyChange(uri, null);

        Log.d(TAG, "Registros borrados: " + ret);
        return ret;
    }

    @Override
    public int update(@NonNull Uri uri, @Nullable ContentValues contentValues, @Nullable String s, @Nullable String[] strings) {
        String where;

        switch (sUriMatcher.match(uri)) {
            case MotoContract.MOTO_DIR:
                where = s;
                break;
            case MotoContract.MOTO_ITEM:
                long id = ContentUris.parseId(uri);
                where = MotoContract.Column.ID
                        + "="
                        + id
                        + (TextUtils.isEmpty(s) ? "" : " and (" + s + ")");
                break;
            default:
                throw new IllegalArgumentException("Uri incorrecta: " + uri);
        }
        SQLiteDatabase db = dbHelper.getWritableDatabase();
        int ret = db.update(MotoContract.TABLE, contentValues, where, strings);
        if (ret > 0)
            getContext().getContentResolver().notifyChange(uri, null);

        Log.d(TAG, "Registros actualizados: " + ret);
        return ret;
    }
}
