package com.example.angelus9110.routes4riders.sqlite;

import android.content.ContentProvider;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.UriMatcher;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.net.Uri;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.text.TextUtils;
import android.util.Log;

public class PointProvider extends ContentProvider {
    private static final String TAG = PointContract.class.getSimpleName();
    private DbHelper dbHelper;
    private static final UriMatcher sUriMatcher;

    static {
        sUriMatcher = new UriMatcher(UriMatcher.NO_MATCH);
        sUriMatcher.addURI(PointContract.AUTHORITY, PointContract.TABLE, PointContract.POINT_DIR);
        sUriMatcher.addURI(PointContract.AUTHORITY, PointContract.TABLE + "/#", PointContract.POINT_ITEM);
    }

    @Override
    public boolean onCreate() {
        dbHelper = new DbHelper(getContext());
        Log.d(TAG, "onCreated");
        return true;
    }

    @Nullable
    @Override
    public Cursor query(@NonNull Uri uri, @Nullable String[] strings, @Nullable String s, @Nullable String[] strings1, @Nullable String s1) {
        String where;

        switch (sUriMatcher.match(uri)) {
            case PointContract.POINT_DIR:
                where = s;
                break;
            case PointContract.POINT_ITEM:
                long id = ContentUris.parseId(uri);
                where = PointContract.Column.CREATED_AT
                        + "="
                        + id
                        + (TextUtils.isEmpty(s) ? "" : " and (" + s + ")");
                break;
            default:
                throw new IllegalArgumentException("Uri incorrecta: " + uri);
        }
        String orderBy = TextUtils.isEmpty(s1) ? PointContract.DEFAULT_SORT : s1;
        SQLiteDatabase db = dbHelper.getReadableDatabase();
        Cursor cursor = db.query(PointContract.TABLE, strings, where, strings1, null, null, orderBy);
        cursor.setNotificationUri(getContext().getContentResolver(), uri);
        Log.d(TAG, "Registros recuperados: " + cursor.getCount());
        return cursor;
    }

    @Nullable
    @Override
    public String getType(@NonNull Uri uri) {
        String type;
        switch (sUriMatcher.match(uri)) {
            case PointContract.POINT_DIR:
                type = "vnd.android.cursor.dir/vnd.com.example.angelus9110.routes4riders.sqlite.provider.point";
                return type;
            case PointContract.POINT_ITEM:
                type = "vnd.android.cursor.item/vnd.com.example.angelus9110.routes4riders.sqlite.provider.point";
                return type;
            default:
                throw new IllegalArgumentException("Uri incorrecta: " + uri);
        }
    }

    @Nullable
    @Override
    public Uri insert(@NonNull Uri uri, @Nullable ContentValues contentValues) {
        Uri ret = null;

        // Aseguramos que la URI es correcta
        if (sUriMatcher.match(uri) != PointContract.POINT_DIR)
            throw new IllegalArgumentException("Uri incorrecta: " + uri);

        SQLiteDatabase db = dbHelper.getWritableDatabase();
        long rowId = db.insertWithOnConflict(PointContract.TABLE, null, contentValues, SQLiteDatabase.CONFLICT_IGNORE);

        if (rowId != -1) {
            long id = contentValues.getAsLong(PointContract.Column.CREATED_AT);
            ret = ContentUris.withAppendedId(uri, id);
            Log.d(TAG, "Uri insertada: " + ret);
            getContext().getContentResolver().notifyChange(uri, null);
        }
        return ret;
    }

    @Override
    public int delete(@NonNull Uri uri, @Nullable String s, @Nullable String[] strings) {
        String where;

        switch (sUriMatcher.match(uri)) {
            case PointContract.POINT_DIR:
                where = s;
                break;
            case PointContract.POINT_ITEM:
                long id = ContentUris.parseId(uri);
                where = PointContract.Column.CREATED_AT
                        + "="
                        + id
                        + (TextUtils.isEmpty(s) ? "" : " and (" + s + ")");
                break;
            default:
                throw new IllegalArgumentException("Uri incorrecta: " + uri);
        }
        SQLiteDatabase db = dbHelper.getWritableDatabase();
        int ret = db.delete(PointContract.TABLE, where, strings);
        if (ret > 0)
            getContext().getContentResolver().notifyChange(uri, null);

        Log.d(TAG, "Registros borrados: " + ret);
        return ret;
    }

    @Override
    public int update(@NonNull Uri uri, @Nullable ContentValues contentValues, @Nullable String s, @Nullable String[] strings) {
        String where;

        switch (sUriMatcher.match(uri)) {
            case PointContract.POINT_DIR:
                where = s;
                break;
            case PointContract.POINT_ITEM:
                long id = ContentUris.parseId(uri);
                where = PointContract.Column.CREATED_AT
                        + "="
                        + id
                        + (TextUtils.isEmpty(s) ? "" : " and (" + s + ")");
                break;
            default:
                throw new IllegalArgumentException("Uri incorrecta: " + uri);
        }
        SQLiteDatabase db = dbHelper.getWritableDatabase();
        int ret = db.update(PointContract.TABLE, contentValues, where, strings);
        if (ret > 0)
            getContext().getContentResolver().notifyChange(uri, null);

        Log.d(TAG, "Registros actualizados: " + ret);
        return ret;
    }
}
