package com.example.angelus9110.routes4riders.ui;

import android.content.Intent;
import android.database.Cursor;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.v4.app.ListFragment;
import android.support.v4.app.LoaderManager;
import android.support.v4.content.CursorLoader;
import android.support.v4.content.Loader;
import android.text.format.DateFormat;
import android.util.Log;
import android.view.View;
import android.widget.ListView;
import android.widget.SimpleCursorAdapter;
import android.widget.TextView;

import com.example.angelus9110.routes4riders.R;
import com.example.angelus9110.routes4riders.sqlite.RouteContract;

import java.util.Calendar;
import java.util.Locale;

public class RouteListFragment extends ListFragment implements LoaderManager.LoaderCallbacks<Cursor> {
    private static final String TAG = RouteListFragment.class.getSimpleName();

    private SimpleCursorAdapter adapter;

    private static final String[] FROM = {RouteContract.Column.CREATED_AT};
    private static final int[] TO = {R.id.list_item_created_at_route};

    private static final int LOADER_ID = 43;

    @Override
    public void onActivityCreated(@Nullable Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        setEmptyText(getContext().getString(R.string.no_data));
        adapter = new SimpleCursorAdapter(getActivity(), R.layout.list_route, null, FROM, TO, 0);

        adapter.setViewBinder(new RouteViewBinder());

        setListAdapter(adapter);
        getLoaderManager().initLoader(LOADER_ID, null, this);
    }

    @Override
    public Loader<Cursor> onCreateLoader(int id, Bundle args) {
        if (id != LOADER_ID)
            return null;
        Log.d(TAG, "onCreateLoader");
        return new CursorLoader(getActivity(), RouteContract.CONTENT_URI, null, null, null, RouteContract.DEFAULT_SORT);
    }

    @Override
    public void onLoadFinished(Loader<Cursor> loader, Cursor cursor) {
        Log.d(TAG, "onLoadFinished with cursor: " + cursor.getCount());
        adapter.swapCursor(cursor);
    }

    @Override
    public void onLoaderReset(Loader<Cursor> loader) {
        Log.d(TAG, "onLoaderReset");
        adapter.swapCursor(null);
    }

    @Override
    public void onListItemClick(ListView l, View v, int position, long id) {
        super.onListItemClick(l, v, position, id);
        Log.d(TAG, "onListItemClicked id: " + id + " position: " + position);
        Cursor routeSelected = (Cursor) getListView().getItemAtPosition(position);
        int routeIdSelected = routeSelected.getInt(routeSelected.getColumnIndex(RouteContract.Column.CREATED_AT));

        Intent intent = new Intent(getContext(), RouteMapsActivity.class);
        intent.putExtra("routeId", routeIdSelected);
        startActivity(intent);
    }

    private class RouteViewBinder implements SimpleCursorAdapter.ViewBinder {
        @Override
        public boolean setViewValue(View view, Cursor cursor, int i) {
            if (view.getId() != R.id.list_item_created_at_route)
                return false;

            // Convertimos el timestamp a tiempo relativo
            long timestamp = cursor.getLong(i);
            String relativeTime = getDate(timestamp);
            ((TextView) view).setText(relativeTime);
            return true;
        }

        private String getDate(long timestamp) {
            Calendar cal = Calendar.getInstance(Locale.ENGLISH);
            cal.setTimeInMillis(timestamp * 1000L);
            String date = DateFormat.format("dd/MM/yyyy HH:mm:ss", cal).toString();
            return date;
        }
    }
}
