package com.example.angelus9110.routes4riders.ui;

import android.content.ActivityNotFoundException;
import android.content.Intent;
import android.database.Cursor;
import android.graphics.Color;
import android.net.Uri;
import android.os.Bundle;
import android.support.v4.app.FragmentActivity;
import android.view.View;
import android.widget.Button;
import android.widget.Toast;

import com.example.angelus9110.routes4riders.R;
import com.example.angelus9110.routes4riders.sqlite.PointContract;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.android.gms.maps.model.Polyline;
import com.google.android.gms.maps.model.PolylineOptions;

import java.util.ArrayList;

public class RouteMapsActivity extends FragmentActivity implements OnMapReadyCallback {
    private static final String TAG = RouteMapsActivity.class.getSimpleName();

    private static final float DEFAULT_ZOOM = 15;

    private GoogleMap mMap;
    Button mapsNavegacionButton;
    Button irInicioRuta;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_route_maps);
        // Obtain the SupportMapFragment and get notified when the map is ready to be used.
        SupportMapFragment mapFragment = (SupportMapFragment) getSupportFragmentManager()
                .findFragmentById(R.id.map);
        mapFragment.getMapAsync(this);

    }


    /**
     * Manipulates the map once available.
     * This callback is triggered when the map is ready to be used.
     * If Google Play services is not installed on the device, the user will be prompted to install
     * it inside the SupportMapFragment. This method will only be triggered once the user has
     * installed Google Play services and returned to the app.
     */
    @Override
    public void onMapReady(GoogleMap googleMap) {
        mMap = googleMap;

        Intent intent = getIntent();
        int routeId = intent.getIntExtra("routeId", -1);
        if (routeId == -1) {
            Toast.makeText(this, R.string.failure_receiving_extra, Toast.LENGTH_LONG);
            return;
        }

        Uri uri = PointContract.CONTENT_URI;
        String where = PointContract.Column.ROUTE_ID + " = " + Integer.toString(routeId);
        Cursor cursor = getContentResolver().query(uri, null, where, null, null);

        if (cursor.getCount() == 0) {
            Toast.makeText(this, R.string.ruta_vacia, Toast.LENGTH_LONG).show();
            return;
        }

        final ArrayList<LatLng> listPoints = new ArrayList<LatLng>();
        while (cursor.moveToNext()) {
            double lat = cursor.getDouble(cursor.getColumnIndex(PointContract.Column.LAT));
            double lon = cursor.getDouble(cursor.getColumnIndex(PointContract.Column.LON));
            listPoints.add(new LatLng(lat, lon));
        }

        mMap.addMarker(new MarkerOptions().position(listPoints.get(0)).title("Final de ruta"));
        mMap.addMarker(new MarkerOptions().position(listPoints.get(listPoints.size() - 1)).title("Inicio de ruta"));

        Polyline line = mMap.addPolyline(new PolylineOptions()
                .addAll(listPoints)
                .width(5)
                .color(Color.RED));
        line.setVisible(true);
        mMap.moveCamera(CameraUpdateFactory.newLatLngZoom(listPoints.get(0), DEFAULT_ZOOM));

        mapsNavegacionButton = findViewById(R.id.btnGoogleMaps);

        mapsNavegacionButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Uri gmmIntentUri = Uri.parse("https://www.google.com/maps/dir/?api=1&origin="
                        +listPoints.get(listPoints.size() - 1).toString()
                        .replace("lat/lng: (","")
                        .replace(")","")+"&destination="
                        +listPoints.get(0).toString()
                        .replace("lat/lng: (","")
                        .replace(")","")+"&travelmode=driving");
                Intent intent = new Intent(Intent.ACTION_VIEW, gmmIntentUri);
                intent.setPackage("com.google.android.apps.maps");
                try {
                    startActivity(intent);
                } catch (ActivityNotFoundException ex) {
                    try {
                        Intent unrestrictedIntent = new Intent(Intent.ACTION_VIEW, gmmIntentUri);
                        startActivity(unrestrictedIntent);
                    } catch (ActivityNotFoundException innerEx) {
                        Toast.makeText(getApplication(), R.string.instalar_maps, Toast.LENGTH_LONG).show();
                    }
                }
            }
        });

        irInicioRuta = findViewById(R.id.btnIrInicioRuta);

        irInicioRuta.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent intent = new Intent(Intent.ACTION_VIEW);
                intent.setData(Uri.parse("https://www.google.com/maps/dir/?api=1&destination="
                        +(listPoints.get(listPoints.size() - 1)).toString()
                        .replace("lat/lng: (","")
                        .replace(")","")+"&travelmode=driving"));
                startActivity(intent);
            }
        });
    }
}
